// This will store the stream id
let streamId = null;

overwolf.streaming.onStartStreaming.addListener(console.log); //register to the onStartStreaming
overwolf.streaming.onStopStreaming.addListener(console.log); //register to the onStopStreaming
overwolf.streaming.onStreamingError.addListener(function(resp) {
  console.log("resp", resp);
  if(!resp.success) {
    streamId = null;
  }
}); //register to the onStreamingError
overwolf.streaming.onStreamingWarning.addListener(console.log); //register to the onStreamingWarning

// Basic settings
const streamSettings = {
  provider: overwolf.streaming.enums.StreamingProvider.RTMP,
  settings: { 
    "audio": {  "mic": { "volume": 100,  "enable": true },
      "game": { "volume": 75, "enable": true },
    },
    "video": {
      "game_window_capture": {
        "enable_when_available": true,    // Disabled by default
        "capture_overwolf_windows": true  // Default value is taken from the Overwolf Settings
      },
      "auto_detect": true,
      "auto_calc_kbps": false,
      "override_overwolf_setting": true,
      "fps": 35,
      "width": 1920,
      "height": 1080,
      "max_kbps": 8000,
      "buffer_length": 20000,
      "max_file_size_bytes": 5000000000, 
      "include_full_size_video": true,
      "notify_dropped_frames_ratio": 0.5,
      "test_drop_frames_interval": 5000,
      "indication_position": "TopLeftCorner",
      "indication_type": "DotAndTimer",
      "base_frame_size_source" : overwolf.streaming.enums.eVideoBaseFrameSizeSource.Auto, 
      "frame_size_method": overwolf.streaming.enums.eVideoFrameSizeCalcMethod.Original,
      "encoder": {
        "name": overwolf.streaming.enums.StreamEncoder.X264,
        "enabled" : true,
        "config": {
          "preset": overwolf.streaming.enums.StreamEncoderPreset_x264.VERYFAST,
          "rate_control": overwolf.streaming.enums.StreamEncoderRateControl_x264.RC_CBR, 
          "keyframe_interval": 2
        }
      }
    },
    "stream_info": {
      "url": "https://discovered.tv/watch?p=Zaj4AGN1",
      "title": "overwolf Stream Discovered Test"
    },
    "auth": {
      "client_id": 1,
      "token": ""
    },
    "ingest_server": {
      "name": "Discovered",
      "template_url": "rtmps://2d0c14bc895d.global-contribute.live-video.net:443/app/sk_us-east-1_mnypRrJoCFHk_ViyGuUbLb9krZ0KX9eQzixqRNGNNIG" 
    }
  },
};

document.getElementById('startRecording').addEventListener('click', () => {
  if (streamId != null) {
    console.log('startRecording(): can\'t start recording because there a stream already recording');
    return;
  }

  overwolf.streaming.start(streamSettings, result => {
    if (!result.success) {
      console.log('overwolf.streaming.start(): error:', result.error, result);
      return;
    }
    
    // Store this for stopping the stream and manipulating stream settings
    streamId = result.stream_id;
  });
});

document.getElementById('stopRecording').addEventListener('click', () => {
  if (streamId === null) {
    console.log('stopRecording(): can\'t stop recording because there is no stream recording');
    return;
  }

  overwolf.streaming.stop(streamId, result => {
    if (result.success) {
      console.log('overwolf.streaming.stop(): success:', result);
      streamId = null;
    } else {
      console.log('overwolf.streaming.stop(): error:', result.error, result);
    }
  });
  
});
